@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2021-2023 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,8 +24,6 @@
 
 #include "rocsparse_importer_rocsparseio.hpp"
 
-#ifdef ROCSPARSEIO
-
 #define ROCSPARSE_CHECK_ROCSPARSEIO(iostatus_)  \
     if(iostatus_ != rocsparseio_status_success) \
     {                                           \
@@ -98,11 +96,18 @@ inline rocsparseio_status rocsparseio2rocsparse_convert(const rocsparseio_index_
 template <typename T>
 inline rocsparseio_type type_tconvert();
 
+template <>
+inline rocsparseio_type type_tconvert<int8_t>()
+{
+    return rocsparseio_type_int8;
+};
+
 template <>
 inline rocsparseio_type type_tconvert<int32_t>()
 {
     return rocsparseio_type_int32;
 };
+
 template <>
 inline rocsparseio_type type_tconvert<int64_t>()
 {
@@ -128,11 +133,10 @@ inline rocsparseio_type type_tconvert<rocsparse_double_complex>()
 {
     return rocsparseio_type_complex64;
 };
-#endif
 
 rocsparse_importer_rocsparseio::~rocsparse_importer_rocsparseio()
 {
-#ifdef ROCSPARSEIO
+
     const char* env = getenv("GTEST_LISTENER");
     if(!env || strcmp(env, "NO_PASS_LINE_IN_LOG"))
     {
@@ -143,13 +147,12 @@ rocsparse_importer_rocsparseio::~rocsparse_importer_rocsparseio()
     if(istatus != rocsparseio_status_success)
     {
     }
-#endif
 }
 
 rocsparse_importer_rocsparseio::rocsparse_importer_rocsparseio(const std::string& filename_)
     : m_filename(filename_)
 {
-#ifdef ROCSPARSEIO
+
     const char* env = getenv("GTEST_LISTENER");
     if(!env || strcmp(env, "NO_PASS_LINE_IN_LOG"))
     {
@@ -163,9 +166,6 @@ rocsparse_importer_rocsparseio::rocsparse_importer_rocsparseio(const std::string
         missing_file_error_message(this->m_filename.c_str());
         throw rocsparse_status_internal_error;
     }
-#else
-    throw rocsparse_status_not_implemented;
-#endif
 }
 
 template <typename I>
@@ -174,10 +174,10 @@ rocsparse_status rocsparse_importer_rocsparseio::import_sparse_coo(I*
                                                                    int64_t*              nnz,
                                                                    rocsparse_index_base* base)
 {
-#ifdef ROCSPARSEIO
-    size_t                 iM;
-    size_t                 iN;
-    size_t                 innz;
+
+    uint64_t               iM;
+    uint64_t               iN;
+    uint64_t               innz;
     rocsparseio_index_base ibase;
     rocsparseio_status     istatus;
     istatus = rocsparseiox_read_metadata_sparse_coo(this->m_handle,
@@ -206,19 +206,15 @@ rocsparse_status rocsparse_importer_rocsparseio::import_sparse_coo(I*
     this->m_nnz = innz;
     ROCSPARSE_CHECK_ROCSPARSEIO(rocsparseio2rocsparse_convert(ibase, *base));
     return rocsparse_status_success;
-#else
-    return rocsparse_status_not_implemented;
-#endif
 }
 
 template <typename T, typename I>
 rocsparse_status rocsparse_importer_rocsparseio::import_sparse_coo(I* row_ind, I* col_ind, T* val)
 {
-#ifdef ROCSPARSEIO
     rocsparseio_status     istatus;
     const rocsparseio_type csr_ind_type = type_tconvert<I>(), csr_val_type = type_tconvert<T>();
 
-    const size_t NNZ = this->m_nnz;
+    const uint64_t NNZ = this->m_nnz;
 
     const bool same_ind_type = (this->m_ind_type == csr_ind_type);
     const bool same_val_type = (this->m_val_type == csr_val_type);
@@ -240,7 +236,7 @@ rocsparse_status rocsparse_importer_rocsparseio::import_sparse_coo(I* row_ind, I
 
         host_dense_vector<char> tmp_row_indv, tmp_col_indv, tmp_valv;
 
-        size_t sizeof_ind_type, sizeof_val_type;
+        uint64_t sizeof_ind_type, sizeof_val_type;
 
         istatus = rocsparseio_type_get_size(this->m_ind_type, &sizeof_ind_type);
         ROCSPARSE_CHECK_ROCSPARSEIO(istatus);
@@ -286,6 +282,7 @@ rocsparse_status rocsparse_importer_rocsparseio::import_sparse_coo(I* row_ind, I
                 break;
             }
 
+            case rocsparseio_type_int8:
             case rocsparseio_type_float32:
             case rocsparseio_type_float64:
             case rocsparseio_type_complex32:
@@ -306,6 +303,12 @@ rocsparse_status rocsparse_importer_rocsparseio::import_sparse_coo(I* row_ind, I
                 break;
             }
 
+            case rocsparseio_type_int8:
+            {
+                rocsparse_importer_copy_mixed_arrays(NNZ, val, (const int8_t*)tmp_val);
+                break;
+            }
+
             case rocsparseio_type_float32:
             {
                 //
@@ -325,6 +328,7 @@ rocsparse_status rocsparse_importer_rocsparseio::import_sparse_coo(I* row_ind, I
             {
                 rocsparse_importer_copy_mixed_arrays(
                     NNZ, val, (const rocsparse_float_complex*)tmp_val);
+                break;
             }
             case rocsparseio_type_complex64:
             {
@@ -336,9 +340,6 @@ rocsparse_status rocsparse_importer_rocsparseio::import_sparse_coo(I* row_ind, I
         }
     }
     return rocsparse_status_success;
-#else
-    return rocsparse_status_not_implemented;
-#endif
 }
 
 template <typename I, typename J>
@@ -351,18 +352,16 @@ rocsparse_status rocsparse_importer_rocsparseio::import_sparse_gebsx(rocsparse_d
                                                                      J* block_dim_column,
                                                                      rocsparse_index_base* base)
 {
-#ifdef ROCSPARSEIO
     rocsparseio_status     istatus;
     rocsparseio_direction  idir, idirb;
     rocsparseio_index_base ibase;
     istatus = rocsparseio_open(&this->m_handle, rocsparseio_rwmode_read, this->m_filename.c_str());
     ROCSPARSE_CHECK_ROCSPARSEIO(istatus);
 
-    size_t iMb;
-    size_t iNb;
-    size_t innzb;
-    size_t irow_block_dim, icol_block_dim;
-
+    uint64_t iMb;
+    uint64_t iNb;
+    uint64_t innzb;
+    uint64_t irow_block_dim, icol_block_dim;
     istatus = rocsparseiox_read_metadata_sparse_gebsx(this->m_handle,
                                                       &idir,
                                                       &idirb,
@@ -404,23 +403,20 @@ rocsparse_status rocsparse_importer_rocsparseio::import_sparse_gebsx(rocsparse_d
     this->m_row_block_dim = irow_block_dim;
     this->m_col_block_dim = icol_block_dim;
     return rocsparse_status_success;
-#else
-    return rocsparse_status_not_implemented;
-#endif
 }
 
 template <typename T, typename I, typename J>
 rocsparse_status rocsparse_importer_rocsparseio::import_sparse_gebsx(I* ptr, J* ind, T* val)
 {
-#ifdef ROCSPARSEIO
+
     rocsparseio_status     istatus;
     const rocsparseio_type csr_ptr_type = type_tconvert<I>(), csr_ind_type = type_tconvert<J>(),
                            csr_val_type = type_tconvert<T>();
 
-    const size_t MB            = this->m_mb;
-    const size_t NNZB          = this->m_nnzb;
-    const size_t row_block_dim = this->m_row_block_dim;
-    const size_t col_block_dim = this->m_col_block_dim;
+    const uint64_t MB            = this->m_mb;
+    const uint64_t NNZB          = this->m_nnzb;
+    const uint64_t row_block_dim = this->m_row_block_dim;
+    const uint64_t col_block_dim = this->m_col_block_dim;
 
     const bool same_ptr_type = (this->m_ptr_type == csr_ptr_type);
     const bool same_ind_type = (this->m_ind_type == csr_ind_type);
@@ -443,7 +439,7 @@ rocsparse_status rocsparse_importer_rocsparseio::import_sparse_gebsx(I* ptr, J*
 
         host_dense_vector<char> tmp_ptrv, tmp_indv, tmp_valv;
 
-        size_t sizeof_ptr_type, sizeof_ind_type, sizeof_val_type;
+        uint64_t sizeof_ptr_type, sizeof_ind_type, sizeof_val_type;
 
         istatus = rocsparseio_type_get_size(this->m_ptr_type, &sizeof_ptr_type);
         ROCSPARSE_CHECK_ROCSPARSEIO(istatus);
@@ -488,6 +484,7 @@ rocsparse_status rocsparse_importer_rocsparseio::import_sparse_gebsx(I* ptr, J*
                 break;
             }
 
+            case rocsparseio_type_int8:
             case rocsparseio_type_float32:
             case rocsparseio_type_float64:
             case rocsparseio_type_complex32:
@@ -512,6 +509,7 @@ rocsparse_status rocsparse_importer_rocsparseio::import_sparse_gebsx(I* ptr, J*
                 rocsparse_importer_copy_mixed_arrays(NNZB, ind, (const int64_t*)tmp_ind);
                 break;
             }
+            case rocsparseio_type_int8:
             case rocsparseio_type_float32:
             case rocsparseio_type_float64:
             case rocsparseio_type_complex32:
@@ -532,6 +530,13 @@ rocsparse_status rocsparse_importer_rocsparseio::import_sparse_gebsx(I* ptr, J*
                 break;
             }
 
+            case rocsparseio_type_int8:
+            {
+                rocsparse_importer_copy_mixed_arrays(
+                    NNZB * row_block_dim * col_block_dim, val, (const int8_t*)tmp_val);
+                break;
+            }
+
             case rocsparseio_type_float32:
             {
                 rocsparse_importer_copy_mixed_arrays(
@@ -551,6 +556,7 @@ rocsparse_status rocsparse_importer_rocsparseio::import_sparse_gebsx(I* ptr, J*
                 rocsparse_importer_copy_mixed_arrays(NNZB * row_block_dim * col_block_dim,
                                                      val,
                                                      (const rocsparse_float_complex*)tmp_val);
+                break;
             }
 
             case rocsparseio_type_complex64:
@@ -565,9 +571,6 @@ rocsparse_status rocsparse_importer_rocsparseio::import_sparse_gebsx(I* ptr, J*
     }
 
     return rocsparse_status_success;
-#else
-    return rocsparse_status_not_implemented;
-#endif
 }
 
 template <typename I, typename J>
@@ -576,16 +579,14 @@ rocsparse_status
     rocsparse_importer_rocsparseio::import_sparse_csx(
         rocsparse_direction* dir, J* m, J* n, I* nnz, rocsparse_index_base* base)
 {
-#ifdef ROCSPARSEIO
     rocsparseio_status     istatus;
     rocsparseio_direction  io_dir;
     rocsparseio_index_base ibase;
     istatus = rocsparseio_open(&this->m_handle, rocsparseio_rwmode_read, this->m_filename.c_str());
     ROCSPARSE_CHECK_ROCSPARSEIO(istatus);
-    size_t iM;
-    size_t iN;
-    size_t innz;
-
+    uint64_t iM;
+    uint64_t iN;
+    uint64_t innz;
     istatus = rocsparseiox_read_metadata_sparse_csx(this->m_handle,
                                                     &io_dir,
                                                     &iM,
@@ -613,21 +614,18 @@ rocsparse_status
     this->m_m   = iM;
     this->m_nnz = innz;
     return rocsparse_status_success;
-#else
-    return rocsparse_status_not_implemented;
-#endif
 }
 
 template <typename T, typename I, typename J>
 rocsparse_status rocsparse_importer_rocsparseio::import_sparse_csx(I* ptr, J* ind, T* val)
 {
-#ifdef ROCSPARSEIO
+
     rocsparseio_status     istatus;
     const rocsparseio_type csr_ptr_type = type_tconvert<I>(), csr_ind_type = type_tconvert<J>(),
                            csr_val_type = type_tconvert<T>();
 
-    const size_t M   = this->m_m;
-    const size_t NNZ = this->m_nnz;
+    const uint64_t M   = this->m_m;
+    const uint64_t NNZ = this->m_nnz;
 
     const bool same_ptr_type = (this->m_ptr_type == csr_ptr_type);
     const bool same_ind_type = (this->m_ind_type == csr_ind_type);
@@ -650,7 +648,7 @@ rocsparse_status rocsparse_importer_rocsparseio::import_sparse_csx(I* ptr, J* in
 
         host_dense_vector<char> tmp_ptrv, tmp_indv, tmp_valv;
 
-        size_t sizeof_ptr_type, sizeof_ind_type, sizeof_val_type;
+        uint64_t sizeof_ptr_type, sizeof_ind_type, sizeof_val_type;
 
         istatus = rocsparseio_type_get_size(this->m_ptr_type, &sizeof_ptr_type);
         ROCSPARSE_CHECK_ROCSPARSEIO(istatus);
@@ -695,6 +693,7 @@ rocsparse_status rocsparse_importer_rocsparseio::import_sparse_csx(I* ptr, J* in
                 break;
             }
 
+            case rocsparseio_type_int8:
             case rocsparseio_type_float32:
             case rocsparseio_type_float64:
             case rocsparseio_type_complex32:
@@ -719,6 +718,7 @@ rocsparse_status rocsparse_importer_rocsparseio::import_sparse_csx(I* ptr, J* in
                 rocsparse_importer_copy_mixed_arrays(NNZ, ind, (const int64_t*)tmp_ind);
                 break;
             }
+            case rocsparseio_type_int8:
             case rocsparseio_type_float32:
             case rocsparseio_type_float64:
             case rocsparseio_type_complex32:
@@ -739,6 +739,12 @@ rocsparse_status rocsparse_importer_rocsparseio::import_sparse_csx(I* ptr, J* in
                 break;
             }
 
+            case rocsparseio_type_int8:
+            {
+                rocsparse_importer_copy_mixed_arrays(NNZ, val, (const int8_t*)tmp_val);
+                break;
+            }
+
             case rocsparseio_type_float32:
             {
                 rocsparse_importer_copy_mixed_arrays(NNZ, val, (const float*)tmp_val);
@@ -755,6 +761,7 @@ rocsparse_status rocsparse_importer_rocsparseio::import_sparse_csx(I* ptr, J* in
             {
                 rocsparse_importer_copy_mixed_arrays(
                     NNZ, val, (const rocsparse_float_complex*)tmp_val);
+                break;
             }
 
             case rocsparseio_type_complex64:
@@ -768,9 +775,6 @@ rocsparse_status rocsparse_importer_rocsparseio::import_sparse_csx(I* ptr, J* in
     }
 
     return rocsparse_status_success;
-#else
-    return rocsparse_status_not_implemented;
-#endif
 }
 
 #define INSTANTIATE_TIJ(T, I, J)                                                             \
