@@ -266,6 +266,25 @@ namespace
                       << rocsparse_datatype2string(arg.compute_type);
                     break;
                 }
+                case rocsparse_test_dispatch_enum::iabct:
+                {
+                    s << rocsparse_indextype2string(arg.index_type_I) << '_'
+                      << rocsparse_datatype2string(arg.a_type) << '_'
+                      << rocsparse_datatype2string(arg.b_type) << '_'
+                      << rocsparse_datatype2string(arg.c_type) << '_'
+                      << rocsparse_datatype2string(arg.compute_type);
+                    break;
+                }
+                case rocsparse_test_dispatch_enum::ijabct:
+                {
+                    s << rocsparse_indextype2string(arg.index_type_I) << '_'
+                      << rocsparse_indextype2string(arg.index_type_J) << '_'
+                      << rocsparse_datatype2string(arg.a_type) << '_'
+                      << rocsparse_datatype2string(arg.b_type) << '_'
+                      << rocsparse_datatype2string(arg.c_type) << '_'
+                      << rocsparse_datatype2string(arg.compute_type);
+                    break;
+                }
                 }
 
                 //
@@ -367,6 +386,56 @@ namespace
         };
     };
 
+    template <rocsparse_test_enum::value_type ROUTINE>
+    struct rocsparse_test_iabct_template
+    {
+        template <typename A, typename B, typename C, typename T, typename I, typename = void>
+        struct test_call : rocsparse_test_invalid
+        {
+        };
+
+        template <typename I, typename A, typename B, typename C, typename T>
+        struct test_call<I, A, B, C, T, typename std::enable_if<std::is_integral<I>::value>::type>
+            : rocsparse_test_template<ROUTINE>::template test_call_proxy<I, A, B, C, T>
+        {
+        };
+
+        struct test : rocsparse_test_template<ROUTINE>::template test_proxy<test, test_call>
+        {
+        };
+    };
+
+    template <rocsparse_test_enum::value_type ROUTINE>
+    struct rocsparse_test_ijabct_template
+    {
+        template <typename A,
+                  typename B,
+                  typename C,
+                  typename T,
+                  typename I,
+                  typename J,
+                  typename = void>
+        struct test_call : rocsparse_test_invalid
+        {
+        };
+
+        template <typename I, typename J, typename A, typename B, typename C, typename T>
+        struct test_call<I,
+                         J,
+                         A,
+                         B,
+                         C,
+                         T,
+                         typename std::enable_if<std::is_integral<I>::value>::type>
+            : rocsparse_test_template<ROUTINE>::template test_call_proxy<I, J, A, B, C, T>
+        {
+        };
+
+        struct test : rocsparse_test_template<ROUTINE>::template test_proxy<test, test_call>
+        {
+        };
+    };
+
     template <rocsparse_test_enum::value_type ROUTINE>
     struct rocsparse_test_ijt_template
     {
